'use strict';
/*
 *  Copyright (C) 1998-2024 by Northwoods Software Corporation. All Rights Reserved.
 */

/**
 * A way of reducing the number of cases where link segments of orthogonal links overlap.
 *
 * Typical setup:
 * ```
 *   myDiagram.routers.add(new LinkLabelRouter());
 * ```
 *
 * If you want to experiment with this extension, try the <a href="../../samples/AvoidsLinksRouter.html">AvoidsLinksRouter</a> sample.
 * @category Router Extension
 */
class LinkLabelRouter extends go.Router {
    constructor(init) {
        super();
        this.name = 'LinkLabelRouter';
        this.isRealtime = false;
        this.layout = new LabelLayout({
            infinityDistance: 100
        });
        if (init)
            Object.assign(this, init);
    }
    /**
     * Only run once per cycle, on the whole Diagram
     */
    canRoute(coll) {
        if (coll instanceof go.Group)
            return false;
        return super.canRoute(coll);
    }
    /**
     * @param {*} coll A Diagram or a Group
     * @param {*} diagram
     * @returns
     */
    routeLinks(links, coll) {
        if (this.layout === null)
            return;
        if (coll instanceof go.Group)
            return;
        this.layout.doLayout(coll.links);
        if (this.layout.network === null)
            return;
        for (const vertex of this.layout.network.vertexes) {
            if (!(vertex instanceof LabelVertex))
                continue;
            const object = vertex.object;
            if (!object)
                continue;
            const x = isNaN(object.alignmentFocus.x) ? 0.5 : object.alignmentFocus.x;
            const y = isNaN(object.alignmentFocus.y) ? 0.5 : object.alignmentFocus.y;
            const dx = vertex.centerX - vertex.startX;
            const dy = vertex.centerY - vertex.startY;
            object.alignmentFocus = new go.Spot(x, y, object.alignmentFocus.offsetX - dx, object.alignmentFocus.offsetY - dy);
        }
    }
}
/** @hidden @internal */
class LabelVertex extends go.ForceDirectedVertex {
    constructor() {
        super(...arguments);
        this.object = null;
        this.startX = 0;
        this.startY = 0;
    }
}
/** @hidden @internal */
class LabelLayout extends go.ForceDirectedLayout {
    isLabel(obj) {
        if (!obj)
            return false;
        const link = obj.panel;
        if (obj.panel === null)
            return false;
        if (link instanceof go.Link) {
            if (obj instanceof go.Shape && (obj.isPanelMain || obj.panel.findMainElement() === obj || obj.fromArrow !== 'None' || obj.toArrow !== 'None')) {
                return false;
            }
            else {
                return true;
            }
        }
        return false;
    }
    /**
     * we should not ever do a prelayout on this virtual, "fake" force-directed network
     */
    needsPrelayout() {
        return false;
    }
    makeNetwork(coll) {
        const net = new go.ForceDirectedNetwork(this);
        let allparts;
        if (coll instanceof go.Diagram) {
            allparts = coll.links;
        }
        else if (coll instanceof go.Group) {
            allparts = coll.memberParts;
        }
        else {
            allparts = coll;
        }
        for (const part of allparts) {
            if (!(part instanceof go.Link))
                continue;
            for (const label of part.elements) {
                if (!this.isLabel(label))
                    continue;
                // add vertex for label node
                const v1 = new LabelVertex(net);
                const documentBounds = label.getDocumentBounds();
                v1.centerX = documentBounds.centerX;
                v1.centerY = documentBounds.centerY;
                v1.width = documentBounds.width;
                v1.height = documentBounds.height;
                v1.object = label;
                v1.charge = 15;
                v1.startX = v1.centerX;
                v1.startY = v1.centerY;
                net.addVertex(v1);
                // add vertex for fixed dummy node
                const v2 = new LabelVertex(net);
                v2.centerX = v1.centerX;
                v2.centerY = v1.centerY;
                v2.charge = 0;
                v2.isFixed = true;
                net.addVertex(v2);
                // add edge
                const e = new go.ForceDirectedEdge(net);
                e.fromVertex = v1;
                e.toVertex = v2;
                net.addEdge(e);
            }
        }
        return net;
    }
}
